@@ -1,7 +1,7 @@
-/* $Id$ */
+/* vi:set et ai sw=2 sts=2 ts=2: */
 /*-
  * Copyright (c) 2005-2006 Benedikt Meurer <benny@xfce.org>
- * Copyright (c) 2009 Jannis Pohlmann <jannis@xfce.org>
+ * Copyright (c) 2009-2012 Jannis Pohlmann <jannis@xfce.org>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -49,6 +49,8 @@
 
 #include <gtk/gtk.h>
 
+#include <exo/exo.h>
+
 #include <libxfce4util/libxfce4util.h>
 
 #include <thunar-uca/thunar-uca-model.h>
@@ -73,6 +75,7 @@ typedef enum
   PARSER_ACTION,
   PARSER_ICON,
   PARSER_NAME,
+  PARSER_UNIQUE_ID,
   PARSER_COMMAND,
   PARSER_STARTUP_NOTIFY,
   PARSER_PATTERNS,
@@ -122,7 +125,7 @@ static gboolean           thunar_uca_model_load_from_file   (ThunarUcaModel
                                                              const gchar          *filename,
                                                              GError              **error);
 static void               thunar_uca_model_item_reset       (ThunarUcaModelItem   *item);
-static void               thunar_uca_model_item_free        (ThunarUcaModelItem   *item);
+static void               thunar_uca_model_item_free        (gpointer              data);
 static void               start_element_handler             (GMarkupParseContext  *context,
                                                              const gchar          *element_name,
                                                              const gchar         **attribute_names,
@@ -160,14 +163,16 @@ struct _ThunarUcaModelItem
 {
   gchar         *name;
   gchar         *description;
-  gchar         *icon;
+  gchar         *unique_id;
+  gchar         *icon_name;
+  GIcon         *gicon;
   gchar         *command;
   guint          startup_notify : 1;
   gchar        **patterns;
   ThunarUcaTypes types;
 
   /* derived attributes */
-  gboolean       multiple_selection : 1;
+  guint          multiple_selection : 1;
 };
 
 typedef XFCE_GENERIC_STACK(ParserState) ParserStack;
@@ -180,13 +185,15 @@ typedef struct
   GString        *name;
   gboolean        name_use;
   guint           name_match;
-  GString        *icon;
+  GString        *unique_id;
+  GString        *icon_name;
   GString        *command;
   GString        *patterns;
   GString        *description;
   gboolean        startup_notify;
   gboolean        description_use;
   guint           description_match;
+  gboolean        unique_id_generated;
   ThunarUcaTypes  types;
 } Parser;
 
@@ -284,8 +291,7 @@ thunar_uca_model_finalize (GObject *object)
   g_object_unref (G_OBJECT (uca_model->icon_factory));
 
   /* release all items */
-  g_list_foreach (uca_model->items, (GFunc) thunar_uca_model_item_free, NULL);
-  g_list_free (uca_model->items);
+  g_list_free_full (uca_model->items, thunar_uca_model_item_free);
 
   (*G_OBJECT_CLASS (thunar_uca_model_parent_class)->finalize) (object);
 }
@@ -317,10 +323,16 @@ thunar_uca_model_get_column_type (GtkTreeModel *tree_model,
     case THUNAR_UCA_MODEL_COLUMN_NAME:
       return G_TYPE_STRING;
 
+    case THUNAR_UCA_MODEL_COLUMN_UNIQUE_ID:
+      return G_TYPE_STRING;
+
     case THUNAR_UCA_MODEL_COLUMN_DESCRIPTION:
       return G_TYPE_STRING;
 
-    case THUNAR_UCA_MODEL_COLUMN_ICON:
+    case THUNAR_UCA_MODEL_COLUMN_GICON:
+      return G_TYPE_ICON;
+
+    case THUNAR_UCA_MODEL_COLUMN_ICON_NAME:
       return G_TYPE_STRING;
 
     case THUNAR_UCA_MODEL_COLUMN_COMMAND:
@@ -410,8 +422,22 @@ thunar_uca_model_get_value (GtkTreeModel *tree_model,
       g_value_set_static_string (value, item->description);
       break;
 
-    case THUNAR_UCA_MODEL_COLUMN_ICON:
-      g_value_set_static_string (value, item->icon);
+    case THUNAR_UCA_MODEL_COLUMN_GICON:
+      if (item->gicon == NULL && item->icon_name != NULL)
+        {
+          /* cache gicon from the name */
+          item->gicon = g_icon_new_for_string (item->icon_name, NULL);
+        }
+
+      g_value_set_object (value, item->gicon);
+      break;
+
+    case THUNAR_UCA_MODEL_COLUMN_ICON_NAME:
+      g_value_set_static_string (value, item->icon_name);
+      break;
+
+    case THUNAR_UCA_MODEL_COLUMN_UNIQUE_ID:
+      g_value_set_static_string (value, item->unique_id);
       break;
 
     case THUNAR_UCA_MODEL_COLUMN_COMMAND:
@@ -555,11 +581,13 @@ thunar_uca_model_load_from_file (ThunarUcaModel *uca_model,
   parser.model = uca_model;
   parser.locale = g_strdup (setlocale (LC_MESSAGES, NULL));
   parser.name = g_string_new (NULL);
-  parser.icon = g_string_new (NULL);
+  parser.unique_id = g_string_new (NULL);
+  parser.icon_name = g_string_new (NULL);
   parser.command = g_string_new (NULL);
   parser.patterns = g_string_new (NULL);
   parser.description = g_string_new (NULL);
   parser.startup_notify = FALSE;
+  parser.unique_id_generated = FALSE;
   xfce_stack_push (parser.stack, PARSER_START);
 
   /* parse the file */
@@ -572,12 +600,18 @@ thunar_uca_model_load_from_file (ThunarUcaModel *uca_model,
   g_string_free (parser.description, TRUE);
   g_string_free (parser.patterns, TRUE);
   g_string_free (parser.command, TRUE);
-  g_string_free (parser.icon, TRUE);
+  g_string_free (parser.icon_name, TRUE);
+  g_string_free (parser.unique_id, TRUE);
   g_string_free (parser.name, TRUE);
   g_free (parser.locale);
   xfce_stack_free (parser.stack);
   g_free (content);
 
+  /* save model to store new ids */
+  if (succeed
+      && parser.unique_id_generated)
+    succeed = thunar_uca_model_save (uca_model, error);
+
   return succeed;
 }
 
@@ -591,7 +625,11 @@ thunar_uca_model_item_reset (ThunarUcaModelItem *item)
   g_free (item->description);
   g_free (item->command);
   g_free (item->name);
-  g_free (item->icon);
+  g_free (item->unique_id);
+  g_free (item->icon_name);
+
+  if (item->gicon != NULL)
+    g_object_unref (item->gicon);
 
   /* ...and reset the item memory */
   memset (item, 0, sizeof (*item));
@@ -600,8 +638,9 @@ thunar_uca_model_item_reset (ThunarUcaModelItem *item)
 
 
 static void
-thunar_uca_model_item_free (ThunarUcaModelItem *item)
+thunar_uca_model_item_free (gpointer data)
 {
+  ThunarUcaModelItem *item = data;
   thunar_uca_model_item_reset (item);
   g_free (item);
 }
@@ -635,8 +674,9 @@ start_element_handler (GMarkupParseContext *context,
           parser->description_match = XFCE_LOCALE_NO_MATCH;
           parser->types = 0;
           parser->startup_notify = FALSE;
-          g_string_truncate (parser->icon, 0);
+          g_string_truncate (parser->icon_name, 0);
           g_string_truncate (parser->name, 0);
+          g_string_truncate (parser->unique_id, 0);
           g_string_truncate (parser->command, 0);
           g_string_truncate (parser->patterns, 0);
           g_string_truncate (parser->description, 0);
@@ -676,9 +716,14 @@ start_element_handler (GMarkupParseContext *context,
 
           xfce_stack_push (parser->stack, PARSER_NAME);
         }
+      else if (strcmp (element_name, "unique-id") == 0)
+        {
+          g_string_truncate (parser->unique_id, 0);
+          xfce_stack_push (parser->stack, PARSER_UNIQUE_ID);
+        }
       else if (strcmp (element_name, "icon") == 0)
         {
-          g_string_truncate (parser->icon, 0);
+          g_string_truncate (parser->icon_name, 0);
           xfce_stack_push (parser->stack, PARSER_ICON);
         }
       else if (strcmp (element_name, "command") == 0)
@@ -799,12 +844,17 @@ end_element_handler (GMarkupParseContext *context,
           thunar_uca_model_append (parser->model, &iter);
           thunar_uca_model_update (parser->model, &iter,
                                    parser->name->str,
+                                   parser->unique_id->str,
                                    parser->description->str,
-                                   parser->icon->str,
+                                   parser->icon_name->str,
                                    parser->command->str,
                                    parser->startup_notify,
                                    parser->patterns->str,
                                    parser->types);
+
+          /* check if a new id should've been generated */
+          if (exo_str_is_empty (parser->unique_id->str))
+            parser->unique_id_generated = TRUE;
         }
       else
         goto unknown_element;
@@ -815,6 +865,11 @@ end_element_handler (GMarkupParseContext *context,
         goto unknown_element;
       break;
 
+    case PARSER_UNIQUE_ID:
+      if (strcmp (element_name, "unique-id") != 0)
+        goto unknown_element;
+      break;
+
     case PARSER_ICON:
       if (strcmp (element_name, "icon") != 0)
         goto unknown_element;
@@ -900,8 +955,12 @@ text_handler (GMarkupParseContext *context,
         g_string_append_len (parser->name, text, text_len);
       break;
 
+    case PARSER_UNIQUE_ID:
+      g_string_append_len (parser->unique_id, text, text_len);
+      break;
+
     case PARSER_ICON:
-      g_string_append_len (parser->icon, text, text_len);
+      g_string_append_len (parser->icon_name, text, text_len);
       break;
 
     case PARSER_COMMAND:
@@ -924,6 +983,19 @@ text_handler (GMarkupParseContext *context,
 
 
 
+static gchar *
+thunar_uca_model_get_unique_id (void)
+{
+  static guint counter = 0;
+
+  /* produce a "<timestamp>-<counter>" string */
+  return g_strdup_printf ("%" G_GINT64_FORMAT "-%d",
+                          g_get_real_time (),
+                          ++counter);
+}
+
+
+
 /**
  * thunar_uca_model_get_default:
  *
@@ -959,6 +1031,8 @@ thunar_uca_model_get_default (void)
 static inline ThunarUcaTypes
 types_from_mime_type (const gchar *mime_type)
 {
+  if (mime_type == NULL)
+    return 0;
   if (strcmp (mime_type, "inode/directory") == 0)
     return THUNAR_UCA_TYPE_DIRECTORIES;
   else if (strncmp (mime_type, "audio/", 6) == 0)
@@ -1000,8 +1074,7 @@ types_from_mime_type (const gchar *mime_type)
  *
  * The caller is responsible to free the returned list using
  * <informalexample><programlisting>
- * g_list_foreach (list, (GFunc) gtk_tree_path_free, NULL);
- * g_list_free (list);
+ * g_list_free_full (list, (GDestroyNotify) gtk_tree_path_free);
  * </programlisting></informalexample>
  *
  * Return value: the list of #GtkTreePath<!---->s to items
@@ -1013,15 +1086,14 @@ thunar_uca_model_match (ThunarUcaModel *uca_model,
 {
   typedef struct
   {
-    const gchar   *name;
-    ThunarUcaTypes types;
+    gchar          *name;
+    ThunarUcaTypes  types;
   } ThunarUcaFile;
 
   ThunarUcaModelItem *item;
   ThunarUcaFile      *files;
-  GFileInfo          *info;
   GFile              *location;
-  const gchar        *mime_type;
+  gchar              *mime_type;
   gboolean            matches;
   GList              *paths = NULL;
   GList              *lp;
@@ -1029,6 +1101,7 @@ thunar_uca_model_match (ThunarUcaModel *uca_model,
   gint                i, m, n;
 
   g_return_val_if_fail (THUNAR_UCA_IS_MODEL (uca_model), NULL);
+  g_return_val_if_fail (file_infos != NULL, NULL);
 
   /* special case to avoid overhead */
   if (G_UNLIKELY (uca_model->items == NULL))
@@ -1051,17 +1124,15 @@ thunar_uca_model_match (ThunarUcaModel *uca_model,
 
       g_object_unref (location);
 
-      info = thunarx_file_info_get_file_info (lp->data);
+      mime_type = thunarx_file_info_get_mime_type (lp->data);
 
-      mime_type = g_file_info_get_content_type (info);
-
-      files[n].name = g_file_info_get_name (info);
+      files[n].name = thunarx_file_info_get_name (lp->data);
       files[n].types = types_from_mime_type (mime_type);
 
       if (G_UNLIKELY (files[n].types == 0))
         files[n].types = THUNAR_UCA_TYPE_OTHER_FILES;
 
-      g_object_unref (info);
+      g_free (mime_type);
     }
 
   /* lookup the matching items */
@@ -1094,6 +1165,8 @@ thunar_uca_model_match (ThunarUcaModel *uca_model,
     }
 
   /* cleanup */
+  for (n = 0; n < n_files; ++n)
+    g_free (files[n].name);
   g_free (files);
 
   return paths;
@@ -1224,6 +1297,7 @@ thunar_uca_model_remove (ThunarUcaModel *uca_model,
  * @uca_model          : a #ThunarUcaModel.
  * @iter               : the #GtkTreeIter of the item to update.
  * @name               : the name of the item.
+ * @unique_id          : a unique ID for the item.
  * @description        : the description of the item.
  * @icon               : the icon for the item.
  * @command            : the command of the item.
@@ -1236,6 +1310,7 @@ void
 thunar_uca_model_update (ThunarUcaModel *uca_model,
                          GtkTreeIter    *iter,
                          const gchar    *name,
+                         const gchar    *unique_id,
                          const gchar    *description,
                          const gchar    *icon,
                          const gchar    *command,
@@ -1258,7 +1333,7 @@ thunar_uca_model_update (ThunarUcaModel *uca_model,
   if (G_LIKELY (name != NULL && *name != '\0'))
     item->name = g_strdup (name);
   if (G_LIKELY (icon != NULL && *icon != '\0'))
-    item->icon = g_strdup (icon);
+    item->icon_name = g_strdup (icon);
   if (G_LIKELY (command != NULL && *command != '\0'))
     item->command = g_strdup (command);
   if (G_LIKELY (description != NULL && *description != '\0'))
@@ -1266,6 +1341,15 @@ thunar_uca_model_update (ThunarUcaModel *uca_model,
   item->types = types;
   item->startup_notify = startup_notify;
 
+  /* set the unique id once */
+  if (item->unique_id == NULL)
+    {
+      if (G_LIKELY (unique_id != NULL && *unique_id != '\0'))
+        item->unique_id = g_strdup (unique_id);
+      else
+        item->unique_id = thunar_uca_model_get_unique_id ();
+    }
+
   /* setup the patterns */
   item->patterns = g_strsplit ((patterns != NULL && *patterns != '\0') ? patterns : "*", ";", -1);
   for (m = n = 0; item->patterns[m] != NULL; ++m)
@@ -1349,11 +1433,13 @@ thunar_uca_model_save (ThunarUcaModel *uca_model,
       patterns = g_strjoinv (";", item->patterns);
       escaped = g_markup_printf_escaped ("\t<icon>%s</icon>\n"
                                          "\t<name>%s</name>\n"
+                                         "\t<unique-id>%s</unique-id>\n"
                                          "\t<command>%s</command>\n"
                                          "\t<description>%s</description>\n"
                                          "\t<patterns>%s</patterns>\n",
-                                         (item->icon != NULL) ? item->icon : "",
+                                         (item->icon_name != NULL) ? item->icon_name : "",
                                          (item->name != NULL) ? item->name : "",
+                                         (item->unique_id != NULL) ? item->unique_id : "",
                                          (item->command != NULL) ? item->command : "",
                                          (item->description != NULL) ? item->description : "",
                                          patterns);
