@@ -1,4 +1,4 @@
-/* $Id$ */
+/* vi:set et ai sw=2 sts=2 ts=2: */
 /*-
  * Copyright (c) 2005-2011 os-cillation e.K.
  *
@@ -103,7 +103,7 @@ thunar_delayed_exit_check (gpointer user_data)
       /* don't call this function again */
       return FALSE;
     }
-  
+
 }
 
 
@@ -139,7 +139,7 @@ main (int argc, char **argv)
   if (!g_thread_supported ())
     g_thread_init (NULL);
 #endif
-    
+
   /* get the startup notification id */
   startup_id = g_getenv ("DESKTOP_STARTUP_ID");
 
@@ -166,7 +166,7 @@ main (int argc, char **argv)
   if (G_UNLIKELY (opt_version))
     {
       g_print ("%s %s (Xfce %s)\n\n", PACKAGE_NAME, PACKAGE_VERSION, xfce_version_string ());
-      g_print ("%s\n", "Copyright (c) 2004-2012");
+      g_print ("%s\n", "Copyright (c) 2004-2015");
       g_print ("\t%s\n\n", _("The Thunar development team. All rights reserved."));
       g_print ("%s\n\n", _("Written by Benedikt Meurer <benny@xfce.org>."));
       g_print (_("Please report bugs to <%s>."), PACKAGE_BUGREPORT);
@@ -185,8 +185,10 @@ main (int argc, char **argv)
     }
 
 #ifdef HAVE_GIO_UNIX
+#if !GLIB_CHECK_VERSION (2, 42, 0)
   /* set desktop environment for app infos */
   g_desktop_app_info_set_desktop_env ("XFCE");
+#endif
 #endif
 
   /* register additional transformation functions */
@@ -302,7 +304,7 @@ error0:
        * schedule an idle source which repeatedly checks whether we are done
        * processing. Once we're done, it'll make the application quit if there
        * are no open windows */
-      g_idle_add_full (G_PRIORITY_LOW, thunar_delayed_exit_check, 
+      g_idle_add_full (G_PRIORITY_LOW, thunar_delayed_exit_check,
                        g_object_ref (application), g_object_unref);
     }
 
