@@ -1,4 +1,4 @@
-/* $Id$ */
+/* vi:set et ai sw=2 sts=2 ts=2: */
 /*-
  * Copyright (c) 2005-2006 Benedikt Meurer <benny@xfce.org>
  *
@@ -27,6 +27,7 @@
 #include <thunar/thunar-abstract-icon-view-ui.h>
 #include <thunar/thunar-gobject-extensions.h>
 #include <thunar/thunar-gtk-extensions.h>
+#include <thunar/thunar-preferences.h>
 #include <thunar/thunar-private.h>
 
 
@@ -44,6 +45,7 @@ static void         thunar_abstract_icon_view_disconnect_ui_manager (ThunarStand
 static GList       *thunar_abstract_icon_view_get_selected_items    (ThunarStandardView           *standard_view);
 static void         thunar_abstract_icon_view_select_all            (ThunarStandardView           *standard_view);
 static void         thunar_abstract_icon_view_unselect_all          (ThunarStandardView           *standard_view);
+static void         thunar_abstract_icon_view_selection_invert      (ThunarStandardView           *standard_view);
 static void         thunar_abstract_icon_view_select_path           (ThunarStandardView           *standard_view,
                                                                      GtkTreePath                  *path);
 static void         thunar_abstract_icon_view_set_cursor            (ThunarStandardView           *standard_view,
@@ -99,13 +101,13 @@ struct _ThunarAbstractIconViewPrivate
   gint ui_merge_id;
 
   /* mouse gesture support */
-  gint gesture_start_x;
-  gint gesture_start_y;
-  gint gesture_current_x;
-  gint gesture_current_y;
-  gint gesture_expose_id;
-  gint gesture_motion_id;
-  gint gesture_release_id;
+  gint   gesture_start_x;
+  gint   gesture_start_y;
+  gint   gesture_current_x;
+  gint   gesture_current_y;
+  gulong gesture_expose_id;
+  gulong gesture_motion_id;
+  gulong gesture_release_id;
 };
 
 
@@ -153,6 +155,7 @@ thunar_abstract_icon_view_class_init (ThunarAbstractIconViewClass *klass)
   thunarstandard_view_class->get_selected_items = thunar_abstract_icon_view_get_selected_items;
   thunarstandard_view_class->select_all = thunar_abstract_icon_view_select_all;
   thunarstandard_view_class->unselect_all = thunar_abstract_icon_view_unselect_all;
+  thunarstandard_view_class->selection_invert = thunar_abstract_icon_view_selection_invert;
   thunarstandard_view_class->select_path = thunar_abstract_icon_view_select_path;
   thunarstandard_view_class->set_cursor = thunar_abstract_icon_view_set_cursor;
   thunarstandard_view_class->scroll_to_path = thunar_abstract_icon_view_scroll_to_path;
@@ -318,6 +321,15 @@ thunar_abstract_icon_view_unselect_all (ThunarStandardView *standard_view)
 
 
 
+static void
+thunar_abstract_icon_view_selection_invert (ThunarStandardView *standard_view)
+{
+  _thunar_return_if_fail (THUNAR_IS_ABSTRACT_ICON_VIEW (standard_view));
+  exo_icon_view_selection_invert (EXO_ICON_VIEW (GTK_BIN (standard_view)->child));
+}
+
+
+
 static void
 thunar_abstract_icon_view_select_path (ThunarStandardView *standard_view,
                                        GtkTreePath        *path)
@@ -406,12 +418,12 @@ thunar_abstract_icon_view_gesture_action (ThunarAbstractIconView *abstract_icon_
   else if (abstract_icon_view->priv->gesture_start_x - abstract_icon_view->priv->gesture_current_x > 40
       && ABS (abstract_icon_view->priv->gesture_start_y - abstract_icon_view->priv->gesture_current_y) < 40)
     {
-      return gtk_ui_manager_get_action (THUNAR_STANDARD_VIEW (abstract_icon_view)->ui_manager, "/main-menu/go-menu/back");
+      return gtk_ui_manager_get_action (THUNAR_STANDARD_VIEW (abstract_icon_view)->ui_manager, "/main-menu/go-menu/placeholder-go-history-actions/back");
     }
   else if (abstract_icon_view->priv->gesture_current_x - abstract_icon_view->priv->gesture_start_x > 40
       && ABS (abstract_icon_view->priv->gesture_start_y - abstract_icon_view->priv->gesture_current_y) < 40)
     {
-      return gtk_ui_manager_get_action (THUNAR_STANDARD_VIEW (abstract_icon_view)->ui_manager, "/main-menu/go-menu/forward");
+      return gtk_ui_manager_get_action (THUNAR_STANDARD_VIEW (abstract_icon_view)->ui_manager, "/main-menu/go-menu/placeholder-go-history-actions/forward");
     }
   else if (abstract_icon_view->priv->gesture_current_y - abstract_icon_view->priv->gesture_start_y > 40
       && ABS (abstract_icon_view->priv->gesture_start_x - abstract_icon_view->priv->gesture_current_x) < 40)
@@ -464,10 +476,13 @@ thunar_abstract_icon_view_button_press_event (ExoIconView            *view,
                                               GdkEventButton         *event,
                                               ThunarAbstractIconView *abstract_icon_view)
 {
-  GtkTreePath *path;
-  GtkTreeIter  iter;
-  ThunarFile  *file;
-  GtkAction   *action;
+  GtkTreePath       *path;
+  GtkTreeIter        iter;
+  ThunarFile        *file;
+  GtkAction         *action;
+  ThunarPreferences *preferences;
+  gboolean           in_tab;
+  const gchar       *action_name;
 
   if (event->type == GDK_BUTTON_PRESS && event->button == 3)
     {
@@ -500,7 +515,7 @@ thunar_abstract_icon_view_button_press_event (ExoIconView            *view,
 
       return TRUE;
     }
-  else if ((event->type == GDK_BUTTON_PRESS || event->type == GDK_2BUTTON_PRESS) && event->button == 2)
+  else if (event->type == GDK_BUTTON_PRESS && event->button == 2)
     {
       /* unselect all currently selected items */
       exo_icon_view_unselect_all (view);
@@ -511,27 +526,28 @@ thunar_abstract_icon_view_button_press_event (ExoIconView            *view,
           /* select only the path to the item on which the user clicked */
           exo_icon_view_select_path (view, path);
 
-          /* if the event was a double-click or we are in single-click mode, then
-           * we'll open the file or folder (folder's are opened in new windows)
-           */
-          if (G_LIKELY (event->type == GDK_2BUTTON_PRESS || exo_icon_view_get_single_click (view)))
+          /* determine the file for the path */
+          gtk_tree_model_get_iter (GTK_TREE_MODEL (THUNAR_STANDARD_VIEW (abstract_icon_view)->model), &iter, path);
+          file = thunar_list_model_get_file (THUNAR_STANDARD_VIEW (abstract_icon_view)->model, &iter);
+          if (G_LIKELY (file != NULL) && thunar_file_is_directory (file))
             {
-              /* determine the file for the path */
-              gtk_tree_model_get_iter (GTK_TREE_MODEL (THUNAR_STANDARD_VIEW (abstract_icon_view)->model), &iter, path);
-              file = thunar_list_model_get_file (THUNAR_STANDARD_VIEW (abstract_icon_view)->model, &iter);
-              if (G_LIKELY (file != NULL))
-                {
-                  /* determine the action to perform depending on the type of the file */
-                  action = thunar_gtk_ui_manager_get_action_by_name (THUNAR_STANDARD_VIEW (abstract_icon_view)->ui_manager,
-                      thunar_file_is_directory (file) ? "open-in-new-window" : "open");
-      
-                  /* emit the action */
-                  if (G_LIKELY (action != NULL))
-                    gtk_action_activate (action);
+              /* lookup setting if we should open in a tab or a window */
+              preferences = thunar_preferences_get ();
+              g_object_get (preferences, "misc-middle-click-in-tab", &in_tab, NULL);
+              g_object_unref (preferences);
 
-                  /* release the file reference */
-                  g_object_unref (G_OBJECT (file));
-                }
+              /* holding ctrl inverts the action */
+              if ((event->state & GDK_CONTROL_MASK) != 0)
+                  in_tab = !in_tab;
+              action_name = in_tab ? "open-in-new-tab" : "open-in-new-window";
+
+              /* emit the action */
+              action = thunar_gtk_ui_manager_get_action_by_name (THUNAR_STANDARD_VIEW (abstract_icon_view)->ui_manager, action_name);
+              if (G_LIKELY (action != NULL))
+                  gtk_action_activate (action);
+
+              /* release the file reference */
+              g_object_unref (G_OBJECT (file));
             }
 
           /* cleanup */
@@ -610,6 +626,7 @@ thunar_abstract_icon_view_expose_event (ExoIconView            *view,
   gchar      *stock_id;
   GdkColor    bg;
   cairo_t    *cr;
+  gint        x, y;
 
   _thunar_return_val_if_fail (EXO_IS_ICON_VIEW (view), FALSE);
   _thunar_return_val_if_fail (THUNAR_IS_ABSTRACT_ICON_VIEW (abstract_icon_view), FALSE);
@@ -617,14 +634,13 @@ thunar_abstract_icon_view_expose_event (ExoIconView            *view,
   _thunar_return_val_if_fail (abstract_icon_view->priv->gesture_motion_id > 0, FALSE);
   _thunar_return_val_if_fail (abstract_icon_view->priv->gesture_release_id > 0, FALSE);
 
-  /* shade the abstract_icon view content while performing mouse gestures */
+  /* create the cairo context (is already clipped) */
   cr = gdk_cairo_create (event->window);
+
+  /* shade the abstract_icon view content while performing mouse gestures */
   bg = GTK_WIDGET (view)->style->base[GTK_STATE_NORMAL];
   cairo_set_source_rgba (cr, bg.red / 65535.0, bg.green / 65535.0, bg.blue / 65535.0, 0.7);
-  cairo_rectangle (cr, event->area.x, event->area.y, event->area.width, event->area.height);
-  cairo_clip (cr);
   cairo_paint (cr);
-  cairo_destroy (cr);
 
   /* determine the gesture action */
   action = thunar_abstract_icon_view_gesture_action (abstract_icon_view);
@@ -646,12 +662,16 @@ thunar_abstract_icon_view_expose_event (ExoIconView            *view,
       /* draw the rendered icon */
       if (G_LIKELY (stock_icon != NULL))
         {
+          /* x/y position of the icon */
+          x = abstract_icon_view->priv->gesture_start_x - gdk_pixbuf_get_width (stock_icon) / 2;
+          y = abstract_icon_view->priv->gesture_start_y - gdk_pixbuf_get_height (stock_icon) / 2;
+
           /* render the stock abstract_icon into the abstract_icon view window */
-          gdk_draw_pixbuf (event->window, NULL, stock_icon, 0, 0,
-                           abstract_icon_view->priv->gesture_start_x - gdk_pixbuf_get_width (stock_icon) / 2,
-                           abstract_icon_view->priv->gesture_start_y - gdk_pixbuf_get_height (stock_icon) / 2,
-                           gdk_pixbuf_get_width (stock_icon), gdk_pixbuf_get_height (stock_icon),
-                           GDK_RGB_DITHER_NONE, 0, 0);
+          gdk_cairo_set_source_pixbuf (cr, stock_icon, x, y);
+          cairo_rectangle (cr, x, y,
+                           gdk_pixbuf_get_width (stock_icon),
+                           gdk_pixbuf_get_height (stock_icon));
+          cairo_fill (cr);
 
           /* release the stock abstract_icon */
           g_object_unref (G_OBJECT (stock_icon));
@@ -661,6 +681,9 @@ thunar_abstract_icon_view_expose_event (ExoIconView            *view,
       g_free (stock_id);
     }
 
+  /* destroy context */
+  cairo_destroy (cr);
+
   return FALSE;
 }
 
