@@ -55,9 +55,12 @@
 #include <thunar/thunar-progress-dialog.h>
 #include <thunar/thunar-renamer-dialog.h>
 #include <thunar/thunar-thumbnail-cache.h>
+#include <thunar/thunar-thumbnailer.h>
 #include <thunar/thunar-util.h>
 #include <thunar/thunar-view.h>
 
+#define ACCEL_MAP_PATH "Thunar/accels.scm"
+
 
 
 /* Prototype for the Thunar job launchers */
@@ -84,6 +87,8 @@ static void           thunar_application_set_property           (GObject
                                                                  guint                   prop_id,
                                                                  const GValue           *value,
                                                                  GParamSpec             *pspec);
+static void           thunar_application_accel_map_changed      (ThunarApplication      *application);
+static gboolean       thunar_application_accel_map_save         (gpointer                user_data);
 static void           thunar_application_collect_and_launch     (ThunarApplication      *application,
                                                                  gpointer                parent,
                                                                  const gchar            *icon_name,
@@ -118,6 +123,7 @@ static void           thunar_application_volman_watch_destroy   (gpointer
 #endif
 static gboolean       thunar_application_show_dialogs           (gpointer                user_data);
 static void           thunar_application_show_dialogs_destroy   (gpointer                user_data);
+static GtkWidget     *thunar_application_get_progress_dialog    (ThunarApplication      *application);
 static void           thunar_application_process_files          (ThunarApplication      *application);
 
 
@@ -136,9 +142,13 @@ struct _ThunarApplication
   GList                 *windows;
 
   ThunarThumbnailCache  *thumbnail_cache;
+  ThunarThumbnailer     *thumbnailer;
 
   gboolean               daemon;
 
+  guint                  accel_map_save_id;
+  GtkAccelMap           *accel_map;
+
   guint                  show_dialogs_timer_id;
 
 #ifdef HAVE_GUDEV
@@ -212,13 +222,12 @@ thunar_application_init (ThunarApplication *application)
 
   /* initialize the application */
   application->preferences = thunar_preferences_get ();
-  application->thumbnail_cache = thunar_thumbnail_cache_new ();
 
   application->files_to_launch = NULL;
   application->progress_dialog = NULL;
 
   /* check if we have a saved accel map */
-  path = xfce_resource_lookup (XFCE_RESOURCE_CONFIG, "Thunar/accels.scm");
+  path = xfce_resource_lookup (XFCE_RESOURCE_CONFIG, ACCEL_MAP_PATH);
   if (G_LIKELY (path != NULL))
     {
       /* load the accel map */
@@ -226,6 +235,11 @@ thunar_application_init (ThunarApplication *application)
       g_free (path);
     }
 
+  /* watch for changes */
+  application->accel_map = gtk_accel_map_get ();
+  g_signal_connect_swapped (G_OBJECT (application->accel_map), "changed",
+      G_CALLBACK (thunar_application_accel_map_changed), application);
+
 #ifdef HAVE_GUDEV
   /* establish connection with udev */
   application->udev_client = g_udev_client_new (subsystems);
@@ -243,21 +257,21 @@ static void
 thunar_application_finalize (GObject *object)
 {
   ThunarApplication *application = THUNAR_APPLICATION (object);
-  gchar             *path;
   GList             *lp;
 
   /* unqueue all files waiting to be processed */
-  thunar_file_list_free (application->files_to_launch);
+  thunar_g_file_list_free (application->files_to_launch);
 
   /* save the current accel map */
-  path = xfce_resource_save_location (XFCE_RESOURCE_CONFIG, "Thunar/accels.scm", TRUE);
-  if (G_LIKELY (path != NULL))
+  if (G_UNLIKELY (application->accel_map_save_id != 0))
     {
-      /* save the accel map */
-      gtk_accel_map_save (path);
-      g_free (path);
+      g_source_remove (application->accel_map_save_id);
+      thunar_application_accel_map_save (application);
     }
 
+  if (application->accel_map != NULL)
+    g_object_unref (G_OBJECT (application->accel_map));
+
 #ifdef HAVE_GUDEV
   /* cancel any pending volman watch source */
   if (G_UNLIKELY (application->volman_watch_id != 0))
@@ -268,8 +282,7 @@ thunar_application_finalize (GObject *object)
     g_source_remove (application->volman_idle_id);
 
   /* drop all pending volume manager UDIs */
-  g_slist_foreach (application->volman_udis, (GFunc) g_free, NULL);
-  g_slist_free (application->volman_udis);
+  g_slist_free_full (application->volman_udis, g_free);
 
   /* disconnect from the udev client */
   g_object_unref (application->udev_client);
@@ -279,6 +292,10 @@ thunar_application_finalize (GObject *object)
   if (G_UNLIKELY (application->show_dialogs_timer_id != 0))
     g_source_remove (application->show_dialogs_timer_id);
 
+  /* drop ref on the thumbnailer */
+  if (application->thumbnailer != NULL)
+    g_object_unref (application->thumbnailer);
+
   /* drop the open windows (this includes the progress dialog) */
   for (lp = application->windows; lp != NULL; lp = lp->next)
     {
@@ -288,7 +305,8 @@ thunar_application_finalize (GObject *object)
   g_list_free (application->windows);
 
   /* release the thumbnail cache */
-  g_object_unref (G_OBJECT (application->thumbnail_cache));
+  if (application->thumbnail_cache != NULL)
+    g_object_unref (G_OBJECT (application->thumbnail_cache));
 
   /* disconnect from the preferences */
   g_object_unref (G_OBJECT (application->preferences));
@@ -342,6 +360,49 @@ thunar_application_set_property (GObject      *object,
 
 
 
+static gboolean
+thunar_application_accel_map_save (gpointer user_data)
+{
+  ThunarApplication *application = THUNAR_APPLICATION (user_data);
+  gchar             *path;
+
+  _thunar_return_val_if_fail (THUNAR_IS_APPLICATION (application), FALSE);
+
+  application->accel_map_save_id = 0;
+
+  /* save the current accel map */
+  path = xfce_resource_save_location (XFCE_RESOURCE_CONFIG, ACCEL_MAP_PATH, TRUE);
+  if (G_LIKELY (path != NULL))
+    {
+      /* save the accel map */
+      gtk_accel_map_save (path);
+      g_free (path);
+    }
+
+  return FALSE;
+}
+
+
+
+static void
+thunar_application_accel_map_changed (ThunarApplication *application)
+{
+  _thunar_return_if_fail (THUNAR_IS_APPLICATION (application));
+
+  /* stop pending save */
+  if (application->accel_map_save_id != 0)
+    {
+      g_source_remove (application->accel_map_save_id);
+      application->accel_map_save_id = 0;
+    }
+
+  /* schedule new save */
+  application->accel_map_save_id =
+      g_timeout_add_seconds (10, thunar_application_accel_map_save, application);
+}
+
+
+
 static void
 thunar_application_collect_and_launch (ThunarApplication *application,
                                        gpointer           parent,
@@ -436,7 +497,7 @@ thunar_application_launch_finished (ThunarJob  *job,
 
   /* the job completed, refresh the interface
    * directly to make it feel snappy */
-  thunar_view_reload (view);
+  thunar_view_reload (view, FALSE);
 }
 
 
@@ -528,6 +589,12 @@ thunar_application_window_destroyed (GtkWidget         *window,
   _thunar_return_if_fail (THUNAR_IS_APPLICATION (application));
   _thunar_return_if_fail (g_list_find (application->windows, window) != NULL);
 
+  /* take a ref on the thumbnailer in daemon mode, this way we don't
+   * need to build the content-type / scheme match table
+   */
+  if (application->thumbnailer == NULL && application->daemon)
+    application->thumbnailer = thunar_thumbnailer_get ();
+
   application->windows = g_list_remove (application->windows, window);
 
   /* terminate the application if we don't have any more
@@ -659,8 +726,7 @@ thunar_application_volman_idle (gpointer user_data)
   else
     {
       /* drop all pending HAL device UDIs */
-      g_slist_foreach (application->volman_udis, (GFunc) g_free, NULL);
-      g_slist_free (application->volman_udis);
+      g_slist_free_full (application->volman_udis, g_free);
       application->volman_udis = NULL;
     }
 
@@ -919,7 +985,7 @@ thunar_application_open_window (ThunarApplication *application,
   gchar     *role;
 
   _thunar_return_val_if_fail (THUNAR_IS_APPLICATION (application), NULL);
-  _thunar_return_val_if_fail (THUNAR_IS_FILE (directory), NULL);
+  _thunar_return_val_if_fail (directory == NULL || THUNAR_IS_FILE (directory), NULL);
   _thunar_return_val_if_fail (screen == NULL || GDK_IS_SCREEN (screen), NULL);
 
   if (G_UNLIKELY (screen == NULL))
@@ -948,7 +1014,8 @@ thunar_application_open_window (ThunarApplication *application,
   gtk_widget_show (window);
 
   /* change the directory */
-  thunar_window_set_current_directory (THUNAR_WINDOW (window), directory);
+  if (directory != NULL)
+    thunar_window_set_current_directory (THUNAR_WINDOW (window), directory);
 
   return window;
 }
@@ -1041,14 +1108,14 @@ thunar_application_bulk_rename (ThunarApplication *application,
 
   /* cleanup */
   g_object_unref (G_OBJECT (current_directory));
-  thunar_file_list_free (file_list);
+  thunar_g_file_list_free (file_list);
 
   return result;
 }
 
 
 
-GtkWidget *
+static GtkWidget *
 thunar_application_get_progress_dialog (ThunarApplication *application)
 {
   _thunar_return_val_if_fail (THUNAR_IS_APPLICATION (application), NULL);
@@ -1103,7 +1170,7 @@ thunar_application_process_files_finish (ThunarBrowser *browser,
         }
 
       /* stop processing files */
-      thunar_file_list_free (application->files_to_launch);
+      thunar_g_file_list_free (application->files_to_launch);
       application->files_to_launch = NULL;
     }
   else
@@ -1231,7 +1298,7 @@ thunar_application_process_filenames (ThunarApplication *application,
                        _("Failed to open \"%s\": %s"), filenames[n], derror->message);
           g_error_free (derror);
 
-          thunar_file_list_free (file_list);
+          thunar_g_file_list_free (file_list);
 
           return FALSE;
         }
@@ -1416,7 +1483,7 @@ thunar_application_create_file (ThunarApplication *application,
       if (is_directory)
         thunar_application_mkdir (application, screen, &path_list, NULL);
       else
-        thunar_application_creat (application, screen, &path_list, NULL);
+        thunar_application_creat (application, screen, &path_list, NULL, NULL);
 
       g_object_unref (path_list.data);
       g_free (name);
@@ -1446,7 +1513,6 @@ thunar_application_create_file_from_template (ThunarApplication *application,
                                               GdkScreen         *screen,
                                               const gchar       *startup_id)
 {
-  GList  source_path_list;
   GList  target_path_list;
   gchar *name;
   gchar *title;
@@ -1470,18 +1536,15 @@ thunar_application_create_file_from_template (ThunarApplication *application,
                                     title);
   if (G_LIKELY (name != NULL))
     {
-      /* fake the source file list */
-      source_path_list.data = thunar_file_get_file (template_file);
-      source_path_list.prev = source_path_list.next = NULL;
-
       /* fake the target path list */
       target_path_list.data = g_file_get_child (thunar_file_get_file (parent_directory), name);
       target_path_list.next = target_path_list.prev = NULL;
 
       /* launch the operation */
-      thunar_application_copy_to (application, screen, 
-                                  &source_path_list, &target_path_list,
-                                  NULL);
+      thunar_application_creat (application, screen,
+                                &target_path_list,
+                                thunar_file_get_file (template_file),
+                                NULL);
 
       /* release the target path */
       g_object_unref (target_path_list.data);
@@ -1691,9 +1754,7 @@ unlink_stub (GList *source_path_list,
  * @application : a #ThunarApplication.
  * @parent      : a #GdkScreen, a #GtkWidget or %NULL.
  * @file_list   : the list of #ThunarFile<!---->s that should be deleted.
- * @permanently : whether to unlink the files permanently. Even if this is set to
- *                FALSE, the files may be erased permanently if the SHIFT key
- *                is pressed at the time the function is called.
+ * @permanently : whether to unlink the files permanently.
  *
  * Deletes all files in the @file_list and takes care of all user interaction.
  *
@@ -1707,25 +1768,18 @@ thunar_application_unlink_files (ThunarApplication *application,
                                  GList             *file_list,
                                  gboolean           permanently)
 {
-  GdkModifierType state;
-  GtkWidget      *dialog;
-  GtkWindow      *window;
-  GdkScreen      *screen;
-  GList          *path_list = NULL;
-  GList          *lp;
-  gchar          *message;      
-  guint           n_path_list = 0;
-  gint            response;
+  GtkWidget *dialog;
+  GtkWindow *window;
+  GdkScreen *screen;
+  GList     *path_list = NULL;
+  GList     *lp;
+  gchar     *message;
+  guint      n_path_list = 0;
+  gint       response;
 
   _thunar_return_if_fail (parent == NULL || GDK_IS_SCREEN (parent) || GTK_IS_WIDGET (parent));
   _thunar_return_if_fail (THUNAR_IS_APPLICATION (application));
 
-  if (!permanently)
-    {
-      /* check if we should permanently delete the files (user holds shift) */
-      permanently = (gtk_get_current_event_state (&state) && (state & GDK_SHIFT_MASK) != 0);
-    }
-
   /* determine the paths for the files */
   for (lp = g_list_last (file_list); lp != NULL; lp = lp->prev, ++n_path_list)
     {
@@ -1828,10 +1882,11 @@ thunar_application_trash (ThunarApplication *application,
 
 
 static ThunarJob *
-creat_stub (GList *source_path_list,
+creat_stub (GList *template_file,
             GList *target_path_list)
 {
-  return thunar_io_jobs_create_files (source_path_list);
+   _thunar_return_val_if_fail (template_file->data == NULL || G_IS_FILE (template_file->data), NULL);
+  return thunar_io_jobs_create_files (target_path_list, template_file->data);
 }
 
 
@@ -1853,15 +1908,21 @@ void
 thunar_application_creat (ThunarApplication *application,
                           gpointer           parent,
                           GList             *file_list,
+                          GFile             *template_file,
                           GClosure          *new_files_closure)
 {
+  GList template_list;
+
   _thunar_return_if_fail (parent == NULL || GDK_IS_SCREEN (parent) || GTK_IS_WIDGET (parent));
   _thunar_return_if_fail (THUNAR_IS_APPLICATION (application));
   
+  template_list.next = template_list.prev = NULL;
+  template_list.data = template_file;
+
   /* launch the operation */
   thunar_application_launch (application, parent, "document-new",
                              _("Creating files..."), creat_stub,
-                             file_list, file_list, new_files_closure);
+                             &template_list, file_list, new_files_closure);
 }
 
 
@@ -2051,6 +2112,10 @@ ThunarThumbnailCache *
 thunar_application_get_thumbnail_cache (ThunarApplication *application)
 {
   _thunar_return_val_if_fail (THUNAR_IS_APPLICATION (application), NULL);
+
+  if (application->thumbnail_cache == NULL)
+    application->thumbnail_cache = thunar_thumbnail_cache_new ();
+
   return g_object_ref (application->thumbnail_cache);
 }
 
