@@ -1,4 +1,4 @@
-/* $Id$ */
+/* vi:set et ai sw=2 sts=2 ts=2: */
 /*-
  * Copyright (c) 2006-2007 Benedikt Meurer <benny@xfce.org>
  *
@@ -117,7 +117,6 @@ thunar_column_editor_init (ThunarColumnEditor *column_editor)
   /* setup the dialog */
   gtk_dialog_add_button (GTK_DIALOG (column_editor), GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE);
   gtk_dialog_set_default_response (GTK_DIALOG (column_editor), GTK_RESPONSE_CLOSE);
-  gtk_dialog_set_has_separator (GTK_DIALOG (column_editor), FALSE);
   gtk_window_set_resizable (GTK_WINDOW (column_editor), FALSE);
   gtk_window_set_title (GTK_WINDOW (column_editor), _("Configure Columns in the Detailed List View"));
 
@@ -307,7 +306,7 @@ thunar_column_editor_help_clicked (GtkWidget          *button,
   xfce_dialog_show_help (GTK_WINDOW (gtk_widget_get_toplevel (button)),
                          "thunar",
                          "the-file-manager-window",
-                         "visible-columns-in-the-detailed-list-view");
+                         "customizing_the_appearance");
 }
 
 
@@ -513,21 +512,6 @@ thunar_column_editor_use_defaults (GtkWidget          *button,
 
 
 
-/**
- * thunar_column_editor_new:
- *
- * Allocates a new #ThunarColumnEditor instance.
- *
- * Return value: the newly allocated #ThunarColumnEditor.
- **/
-GtkWidget*
-thunar_column_editor_new (void)
-{
-  return g_object_new (THUNAR_TYPE_COLUMN_EDITOR, NULL);
-}
-
-
-
 /**
  * thunar_show_column_editor:
  * @parent : the #GtkWidget the #GdkScreen on which to open the
@@ -569,10 +553,10 @@ thunar_show_column_editor (gpointer parent)
     }
 
   /* display the column editor */
-  dialog = thunar_column_editor_new ();
+  dialog = g_object_new (THUNAR_TYPE_COLUMN_EDITOR, NULL);
 
   /* check if we have a toplevel window */
-  if (G_LIKELY (window != NULL && GTK_WIDGET_TOPLEVEL (window)))
+  if (G_LIKELY (window != NULL && gtk_widget_get_toplevel (window)))
     {
       /* dialog is transient for toplevel window and modal */
       gtk_window_set_destroy_with_parent (GTK_WINDOW (dialog), TRUE);
